/*
 * Copyright 2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.openhft.koloboke.collect.impl.hash;

import net.openhft.koloboke.collect.hash.*;
import net.openhft.koloboke.collect.map.ShortObjMap;
import net.openhft.koloboke.collect.map.hash.HashShortObjMapFactory;

import javax.annotation.Nonnull;
import java.util.Map;


public abstract class LHashSeparateKVShortObjMapFactorySO<V>
        extends ShortLHashFactory 
        implements HashShortObjMapFactory<V> {

    LHashSeparateKVShortObjMapFactorySO(HashConfig hashConf, int defaultExpectedSize
            , short lower, short upper) {
        super(hashConf, defaultExpectedSize, lower, upper);
    }

    

    

    


    <V2 extends V> MutableLHashSeparateKVShortObjMapGO<V2> uninitializedMutableMap() {
        return new MutableLHashSeparateKVShortObjMap<V2>();
    }
    <V2 extends V> UpdatableLHashSeparateKVShortObjMapGO<V2> uninitializedUpdatableMap() {
        return new UpdatableLHashSeparateKVShortObjMap<V2>();
    }
    <V2 extends V> ImmutableLHashSeparateKVShortObjMapGO<V2> uninitializedImmutableMap() {
        return new ImmutableLHashSeparateKVShortObjMap<V2>();
    }

    @Override
    @Nonnull
    public <V2 extends V> MutableLHashSeparateKVShortObjMapGO<V2> newMutableMap(int expectedSize) {
        MutableLHashSeparateKVShortObjMapGO<V2> map = uninitializedMutableMap();
        map.init(configWrapper, expectedSize, getFree());
        return map;
    }

    

    @Override
    @Nonnull
    public <V2 extends V> UpdatableLHashSeparateKVShortObjMapGO<V2> newUpdatableMap(int expectedSize) {
        UpdatableLHashSeparateKVShortObjMapGO<V2> map = uninitializedUpdatableMap();
        map.init(configWrapper, expectedSize, getFree());
        return map;
    }

    

    @Override
    @Nonnull
    public <V2 extends V> UpdatableLHashSeparateKVShortObjMapGO<V2> newUpdatableMap(
            Map<Short, ? extends V2> map) {
        if (map instanceof ShortObjMap) {
            if (map instanceof SeparateKVShortObjLHash) {
                SeparateKVShortObjLHash hash = (SeparateKVShortObjLHash) map;
                if (hash.hashConfig().equals(hashConf)) {
                    UpdatableLHashSeparateKVShortObjMapGO<V2> res = uninitializedUpdatableMap();
                    res.copy(hash);
                    return res;
                }
            }
            UpdatableLHashSeparateKVShortObjMapGO<V2> res = newUpdatableMap(map.size());
            res.putAll(map);
            return res;
        }
        UpdatableLHashSeparateKVShortObjMapGO<V2> res = newUpdatableMap(map.size());
        for (Map.Entry<Short, ? extends V2> entry : map.entrySet()) {
            res.put(entry.getKey(), entry.getValue());
        }
        return res;
    }
}

